package com.gihon.security.controller;
import com.gihon.common.http.HttpResult;
import com.gihon.security.mapper.RoleMapper;
import com.gihon.security.pojo.Role;
import com.gihon.security.service.RoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @program: 角色控制器
 * @description:
 * @author: dl
 * @create: 02月18日 15时
 **/
@RestController
public class RoleController {
    @Autowired
    private RoleMapper roleMapper;
    @Autowired
    private RoleService roleService;
    /**
     * 主要功能：
     * @description 根据
     * @author: dl
     * @data 2021/2/20 13:10
     * @param	id	角色id
     * @param	rolename 角色名称
     * @return com.gihon.common.http.HttpResult
     */
    @PreAuthorize("hasAuthority('addRole')")
    @Transactional(rollbackFor = Exception.class)
    @PostMapping(value = "/addRole")
    public HttpResult addRole(Long id, String rolename) {
        Role role = new Role(id, rolename);
        try {
            int insert = roleMapper.insert(role);
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResult.error("添加失败");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 根据Role ID  删除角色
     * @author: dl
     * @data 2021/2/20 13:18
     * @param	id	 Role ID
     * @return com.gihon.common.http.HttpResult
     */
    public HttpResult 删除角色(Long id){
        int i = roleMapper.deleteById(id);
        if (i == 0) {
            return HttpResult.error("添加失败");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 根据 id 输入角色名称修改角色
     * @author: dl
     * @data 2021/2/20 13:25
     * @param	id
     * @param	rolename
     * @return com.gihon.common.http.HttpResult
     */
    public HttpResult setRole(long id,String rolename){
        Role role = Role.builder()
                .id(id)
                .name(rolename)
                .build();
        int update = roleMapper.updateById(role);
        if (update == 0) {
            return HttpResult.error("修改角色名称失败");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 根据ID查找角色信息
     * @author: dl
     * @data 2021/2/20 13:27
     * @param	id
     * @return com.gihon.common.http.HttpResult
     */
    public HttpResult findRole(Long id){
        Role role = roleMapper.selectById(id);
        return HttpResult.ok(role);
    }
    /**
     * 主要功能：
     * @description 根据RoleId 查找对应对应权限
     * @author: dl
     * @data 2021/2/20 14:06
     * @param	id
     * @return com.gihon.common.http.HttpResult
     */
    public HttpResult findPermissionByRoleId(Long id){
        HttpResult<List<Role>> permissionsByRoleId = roleService.findPermissionByRoleId(id);
        return permissionsByRoleId;
    }

}
