package com.gihon.security.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gihon.common.http.HttpResult;
import com.gihon.security.encoder.MyPasswordEncoder;
import com.gihon.security.pojo.User;
import com.gihon.security.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 用户控制器
 *
 */
@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;

    /**
     * 主要功能：分页查询用户列表
     * @data 2021/2/19 9:45
     * @author: dl
     * @description 分页查看用户
     * @param	page    页数
     * @param	rows	页小大
     * @return com.gihon.common.http.HttpResult
     */
    //@Secured(value = {"ROLE_管理员", "ROLE_超级管理员"})
    @GetMapping("/getUsersByPage")
    public HttpResult<User> getUsersByPage(
            @RequestParam(name = "page", defaultValue = "0") int page,
            @RequestParam(name = "page", defaultValue = "10") int rows) {
        HttpResult<User> byPage = userService.findByPage(page, rows);
        return byPage;
    }
    /**
     * 主要功能：查询 用户信息by用户ID
     * @data 2021/2/19 10:05
     * @author: dl
     * @description 根据用户主键
     * 查找用户信息
     * @param	id
     * @return com.gihon.common.http.HttpResult
     *
     */
    @GetMapping("/userByID")
    public HttpResult userFindOne(@RequestParam Long id){
        User userInfo = userService.getById(id);
        return HttpResult.ok(userInfo);
    }
    /**
     * 主要功能：增加用户
     * @data 2021/2/19 9:48
     * @author: dl
     * @description 根据输入用户信息 增加数据
     * 不允许增加重复用户
     * 使用自定义加密器加密用户输入密码后  存储
     *
     * @param	user
     * @return com.gihon.common.http.HttpResult
     *
     */

    //@Secured(value = {"addUser"})
    @PostMapping("/addUser")
    public HttpResult<User> addUser(@RequestBody User user) {
        HttpResult<User> addUser = userService.addUser(user);
        return HttpResult.ok();
    }
    /**
     * 主要功能：删除用户
     * @data 2021/2/19 9:50
     * @author: dl
     * @description 根据用户主键
     * 删除用户信息
     * @param	id
     * @return com.gihon.common.http.HttpResult
     *
     */
    @Transactional(rollbackFor = Exception.class)
    @GetMapping(value = "/deleteUser")
    public HttpResult deleteUser(@RequestParam Long id) {
        boolean b = userService.removeById(id);
        //添加数据失败
        if (!b) {
            return HttpResult.error("删除用户失败");
        }
        return HttpResult.ok();

    }

    @PreAuthorize("hasAuthority('updateUser')")
    @Transactional(rollbackFor = Exception.class)
    @PostMapping(value = "/setUser")
    /**
     * 主要功能：修改密码
     * @data 2021/2/19 9:51
     * @author: dl
     * @description 根据 用户名密码
     * 修改用户密码
     * @param	username
     * @param	password
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     *
     */
    public HttpResult<User> modifyUser(String username, String password) {
        //修改新密码
        MyPasswordEncoder encoder = new MyPasswordEncoder();
        User user = User.builder().username(username).build();
        user.setPassword(encoder.encode(password));
        //根据用户名
        boolean update = userService.update(user, new QueryWrapper<User>()
                .eq("username", user.getUsername())
        );
        //userService.lambdaUpdate().eq(User::getId,1)
        //        .set(User::getId,1)
        //        .set(User::getUsername,"213")
        //        .update();
        //修改结果
        if (!update) {
            return HttpResult.error("修改密码访问数据库失败");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 根据用户id 查找对应权限
     * @author: dl
     * @data 2021/2/20 14:24
     * @param	id	 用户id
     * @return com.gihon.common.http.HttpResult<java.util.List<java.lang.String>>
     */
    @RequestMapping("/getPermissionsByUserId")
    public HttpResult<List<String>> getPermissionsByUserId(Long id){
        List<String> permissions = userService.findPermissionsByUserId(id);
        return HttpResult.ok(permissions);
    }
}
