package com.gihon.security.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.common.http.HttpResult;
import com.gihon.security.encoder.MyPasswordEncoder;
import com.gihon.security.mapper.UserMapper;
import com.gihon.security.pojo.User;
import com.gihon.security.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
/**
 * @program: common-parent
 * @description:
 * @author: dl
 * @create: 02月20日 09时
 **/
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper,User>  implements UserService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private UserService userService;
    /**
     * 主要功能：
     * @description 根据用户主键查找 用户信息
     * @author: dl
     * @data 2021/2/20 11:43
     * @param	id	 用户信息
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     */
    public HttpResult<User> findUserOne(Long id){
        User user = userMapper.selectById(id);
        if (user==null){
            return HttpResult.error("未查找到此用户");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 分页查询用户信息
     * @author: dl
     * @data 2021/2/20 11:38
     * @param	page
     * @param	rows
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     */
    @Override
    public HttpResult<User> findByPage(int page, int rows) {
        //分页配置
        Page<User> itemParamPage = new Page<>(page, rows);
        ////查询条件
        //QueryWrapper<User> Wrapper = new QueryWrapper<>();
        ////根据主键排序
        //Wrapper.orderByAsc("id");
        //得到分页数据
        Page<User> resultPage = (Page<User>) userMapper.selectPage(itemParamPage, null);
        List<User> list = resultPage.getRecords();
        return HttpResult.ok(list);
    }
    /**
     * 主要功能：
     * @description 根据输入信息增加用户
     * @author: dl
     * @data 2021/2/20 11:39
     * @param	user
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     */
    @Override
    public HttpResult<User> addUser(User user) {
        QueryWrapper<User> wrapper = new QueryWrapper<>(user);
        //不允许增加重复用户
        User user1 = userMapper.selectOne(wrapper);
        if (user1 != null) {
            return HttpResult.error("用户已存在");
        }
        //密码加密
        String passWord = new MyPasswordEncoder().encode(user.getPassword());
        user.setPassword(passWord);
        int i = userMapper.insert(user);
        //增加失败
        if (i == 0) {
            return HttpResult.error("添加用户，访问数据库失败");
        }
        return HttpResult.ok();
    }

    @Override
    public List<String> findPermissionsByUserId(Long id) {
        List<String> strings = userMapper.selectPermissionsByUserId(id);
        return strings;
    }

    /**
     * 主要功能：
     * @description 根据主键删除用户
     * @author: dl
     * @data 2021/2/20 11:41
     * @param	id	 用户主键
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     */
    public HttpResult<User> deleteUser(Long id){
        int i = userMapper.deleteById(id);
        if (i == 0) {
            return HttpResult.error("删除用户，访问数据库失败");
        }
        return HttpResult.ok();
    }
    /**
     * 主要功能：
     * @description 根据输入密码和用户id
     * 修改用户信息
     * @author: dl
     * @data 2021/2/20 11:52
     * @param	id
     * @param	password
     * @return com.gihon.common.http.HttpResult<com.gihon.security.pojo.User>
     */
    public HttpResult<User> setPasswordByUserId(Long id,String password){
        User user = User.builder()
                .id(id)
                .password(password)
                .build();
        int update = userMapper.updateById(user);
        if (update == 0) {
            return HttpResult.error("数据库 修改此用户失败");
        }
        return HttpResult.ok();
    }
}
