package com.gihon.common.web.config;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.method.HandlerMethod;

import com.gihon.common.exception.BaseException;
import com.gihon.common.web.response.PageResponse;
import com.gihon.common.web.response.Response;
import com.gihon.common.web.response.ResponseStatus;
/**
 * 基本的全局异常处理器
 * @author baihe
 *
 */

@ControllerAdvice
public class CommonExceptionController {

	private Logger log = LoggerFactory.getLogger(CommonExceptionController.class);
	
	//通过Handler获取到Controller的返回值类型
	private ResponseEntity createResponse(HandlerMethod handler,ResponseStatus status,String message) {
		Class clazz = handler.getMethod().getReturnType();
		Response r = null;
		if(clazz.equals(PageResponse.class)) {
			 r = PageResponse.error(status,message);
		}else {
			r = Response.error(status,message);
		}
		return new ResponseEntity<>(r, HttpStatus.OK);
	}
	
	
	@ExceptionHandler(BaseException.class)
	public Object handleBaseException(HttpServletRequest req,HandlerMethod handler,BaseException e){
		log.warn("["+e.getMessage()+"]",e);
		return createResponse(handler,e.getStatus(),e.getMessage());
	}
	
	@ExceptionHandler(Throwable.class)
	public Object handleThrowable(HttpServletRequest req,HandlerMethod handler,Throwable e){
		log.warn("["+e.getMessage()+"]",e);
		return createResponse(handler,ResponseStatus.ERROR,ResponseStatus.ERROR.getDesc());
	}
	
	/**
	 * 在controller中使用 @Valid or @Validator 抛出的异常
	 * @param req
	 * @param e
	 * @return
	 */
	@ExceptionHandler(MethodArgumentNotValidException.class)
    public Object MethodArgumentNotValidExceptionHandler(HttpServletRequest req,HandlerMethod handler,MethodArgumentNotValidException e) {
		log.warn("["+e.getMessage()+"]",e);
		StringBuilder message = new StringBuilder();
		for (ObjectError error : e.getBindingResult().getAllErrors()) {
			message.append(error.getDefaultMessage()).append("\n");
        }
		
		return createResponse(handler,ResponseStatus.BUSINESS_ERROR,message.toString());
    }
}
