package com.gihon.common.web.config;

import java.text.SimpleDateFormat;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jackson.Jackson2ObjectMapperBuilderCustomizer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.gihon.common.util.DateUtils;

@Configuration
public class WebConfig implements WebMvcConfigurer{
	//SpringBoot 按照JacksonProperties 创建
	//Jackson2ObjectMapperBuilderCustomizerConfiguration.StandardJackson2ObjectMapperBuilderCustomizer
	@Autowired
	private Jackson2ObjectMapperBuilder jackson2ObjectMapperBuilder;
	
	@Bean("jackson2ObjectMapperBuilderCustomizer")
	public Jackson2ObjectMapperBuilderCustomizer jackson2ObjectMapperBuilderCustomizer() {
	    Jackson2ObjectMapperBuilderCustomizer customizer = new Jackson2ObjectMapperBuilderCustomizer() {
	        @Override
	        public void customize(Jackson2ObjectMapperBuilder jacksonObjectMapperBuilder) {
	            jacksonObjectMapperBuilder.serializerByType(Long.class, ToStringSerializer.instance)
	                    .serializerByType(Long.TYPE, ToStringSerializer.instance);
	        }
	    };
	    return customizer;
	}
	
	@Override
	public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {
		ObjectMapper objectMapper = jackson2ObjectMapperBuilder.build();
		objectMapper.configure(MapperFeature.PROPAGATE_TRANSIENT_MARKER, true);// 忽略 transient 修饰的属性
		objectMapper.setDateFormat(new SimpleDateFormat(DateUtils.DATE_TIME));
		
		for(HttpMessageConverter<?> converts:converters) {
			if(converts instanceof MappingJackson2HttpMessageConverter) {
				MappingJackson2HttpMessageConverter convert = (MappingJackson2HttpMessageConverter)converts;
				convert.setObjectMapper(objectMapper);
			}
		}
	}
}
